# Test Time Robustification of Deep Models via Adaptation and Augmentation

These directories contain code for reproducing the MEME results for the CIFAR-10
and ImageNet test sets, as well as CSV files with full results for ImageNet-C.

_Please note:_ this code has been modified from the version that generated the
results in the paper for the purposes of anonymizing and cleaning the code.
Though it is likely that the code is correct and should produce the same
results, it is possible that there may be some discrepancies that were not
caught. Upon publication, we will ensure a full code release that reproduces all
of the results in the paper.


## Setup

First, create a Anaconda environment with `requirements.txt`, e.g.,
```
conda create -n meme python=3.8 -y -q --file requirements.txt
conda activate meme
```

After doing so, you will need to `pip install tqdm`. For the robust vision
transformer models, you will also need to `pip install timm einops`.


## CIFAR-10 Experiments

The `cifar-10-exps` directory contains code for the CIFAR-10 experiments. You
can run `bash script_c10.sh` for the full set of experiments. Alternatively, you
can run `python script_test_c10.py` directly with the experiment you wish to run
(see `script_c10.sh` for more details).

For convenience, we provide the ResNet26 model that we trained in
`results/cifar10_rn26_gn/ckpt.pth`. We cannot provide the datasets due to file
size limitations. You can download the non standard test sets here:

- [CIFAR-10.1](https://github.com/modestyachts/CIFAR-10.1)
- [CIFAR-10-C](https://zenodo.org/record/2535967)

After downloading and setting up the datasets, make sure to modify the
`dataroot` variable on line 8 of `script_test_c10.py`.


## ImageNet Experiments

The `imagenet-exps` directory contains code for the ImageNet experiments. You
can run `bash script_in.sh` for the full set of experiments, though this is very
slow. You can again run `python script_test_in.py` directly with the experiment
you wish to run. For the corrupted image datasets, you may wish to slightly
modify the code to only run one corruption-level pair (and then parallelize).

We have set up the model path structure in `results`, but unlike for CIFAR-10,
we cannot provide the pretrained models due to their size. We did not train our
own models for ImageNet, and all models we used can be downloaded:

- The baseline ResNet-50 is from `torchvision`, and we simply loaded and saved
  out the model weights.
- [ResNet50 w/ DeepAugment + AugMix](https://drive.google.com/file/d/1QKmc\_p6-qDkh51WvsaS9HKFv8bX5jLnP)
- [ResNet50 w/ moment exchange + CutMix](https://drive.google.com/file/d/1cCvhQKV93pY-jj8f5jITywkB9EabiQDA)
- [RVT\*-small](https://drive.google.com/file/d/1g40huqDVthjS2H5sQV3ppcfcWEzn9ekv)

_Please note:_ some of these models provide the weights in slightly different
conventions, thus loading the downloaded `state_dict` may not directly work, and
the keys in the `state_dict` may need to be modified to match with the code.

We also cannot provide the datasets due to file size limitations. You can
download the test sets here:

- [ImageNet-C](https://zenodo.org/record/2235448)
- [ImageNet-R](https://people.eecs.berkeley.edu/~hendrycks/imagenet-r.tar)
- [ImageNet-A](https://people.eecs.berkeley.edu/~hendrycks/imagenet-a.tar)

After downloading and setting up the datasets, again make sure to modify the
`dataroot` variable on line 8 of `script_test_in.py`.


## ImageNet-C Results

The `imagenet-c-results` directory contains comma separated values (CSV) files
detailing full results on ImageNet-C for all methods evaluated in Table 2 of the
paper. Full CIFAR-10-C results are directly provided in Appendix B of the paper.
